# coding: utf-8
from maxipago.managers.base import ManagerRapi
from maxipago.requesters.transaction import TransactionRequester
from maxipago.resources.transaction import TransactionResource

class TransactionManager(ManagerRapi):

    def get(self, **kwargs):
        fields = (
            ('transaction_id', {'translated_name': 'filterOptions/transactionId'}),
        )
        
        requester = TransactionRequester(fields, kwargs)
        return self.send(command='transactionDetailReport', requester=requester, resource=TransactionResource)

    def list(self, **kwargs):
        fields = (
            ('period', {'translated_name': 'filterOptions/period'}),
            ('page_size', {'translated_name': 'filterOptions/pageSize', 'required': False}),
            ('start_date', {'translated_name': 'filterOptions/startDate', 'required': False}),
            ('end_date', {'translated_name': 'filterOptions/endDate', 'required': False}),
            ('start_time', {'translated_name': 'filterOptions/startTime', 'required': False}),
            ('end_time', {'translated_name': 'filterOptions/endTime', 'required': False}),
            ('order_by', {'translated_name': 'filterOptions/orderByName', 'required': False}),
            ('order_direction', {'translated_name': 'filterOptions/orderByDirection', 'required': False}),
            ('offset', {'translated_name': 'filterOptions/startRecordNumber', 'required': False}),
            ('end_record_number', {'translated_name': 'filterOptions/endRecordNumber', 'required': False}),
            ('page_token', {'translated_name': 'filterOptions/pageToken', 'required': False}),
            ('page_number', {'translated_name': 'filterOptions/pageNumber', 'required': False}),
        )
        requester = TransactionRequester(fields, kwargs)
        return self.send(command='transactionDetailReport', requester=requester, resource=TransactionResource)

    def request_status(self, **kwargs):
        fields = (
            ('request_token', {'translated_name': 'requestToken'}),
        )
        requester = TransactionRequester(fields, kwargs)
        return self.send(command='checkRequestStatus', requester=requester)



# coding: utf-8

try:
  from lxml import etree
except ImportError:
  try:
    # Python 2.5
    import xml.etree.cElementTree as etree
  except ImportError:
    try:
      # Python 2.5
      import xml.etree.ElementTree as etree
    except ImportError:
      try:
        # normal cElementTree install
        import cElementTree as etree
      except ImportError:
        try:
          # normal ElementTree install
          import elementtree.ElementTree as etree
        except ImportError:
          print("Failed to import ElementTree from any known place")


def create_element_recursively(parent, path):
    nodes = path.split('/')
    node = parent
    for n_str in nodes:
        n = node.find(n_str)
        if n is None:
            node = etree.SubElement(node, n_str)
        else:
            node = n
    return node



# coding: utf-8
import requests
from maxipago.exceptions import HttpErrorException
from maxipago.utils import etree, create_element_recursively

class Manager(object):
    api_type = None

    def __init__(self, maxid, api_key, api_version, sandbox):
        self.maxid = maxid
        self.api_key = api_key
        self.api_version = api_version
        self.sandbox = sandbox

        if sandbox:
            self.uri_transaction = 'https://testapi.maxipago.net/UniversalAPI/postXML'
            self.uri_api = 'https://testapi.maxipago.net/UniversalAPI/postAPI'
            self.uri_rapi = 'https://testapi.maxipago.net/ReportsAPI/servlet/ReportsAPI'
        else:
            self.uri_transaction = 'https://api.maxipago.net/UniversalAPI/postXML'
            self.uri_api = 'https://api.maxipago.net/UniversalAPI/postAPI'
            self.uri_rapi = 'https://api.maxipago.net/ReportsAPI/servlet/ReportsAPI'
		
    def request(self, xml_data, api_type=None):
        uri = self.get_uri(api_type)
        response = requests.post(url=uri, data=xml_data, headers={'content-type': 'text/xml'})
        
        if not str(response.status_code).startswith('2'):
            raise HttpErrorException(u'Error %s: %s' % (response.status_code, response.reason))
        return response

    def get_uri(self, api_type=None):
        api_type = api_type or self.uri_rapi

        uri_list = {
            'transaction': self.uri_transaction,
            'api': self.uri_api,
            'rapi': self.uri_rapi,
        }
        return uri_list.get(self.api_type)


class ManagerApi(Manager):
    api_type = 'api'

    def send(self, command, params=None, requester=None, api_type=None, resource=None):
        if not params:
            params = requester.translated_data

        root = etree.Element('api-request')
        
        verification = etree.SubElement(root, 'verification')
        etree.SubElement(verification, 'merchantId').text = self.maxid
        etree.SubElement(verification, 'merchantKey').text = self.api_key

        etree.SubElement(root, 'command').text = command

        request = etree.SubElement(root, 'request')

        for key, value in params:
            create_element_recursively(request, key).text = value

        xml_data = etree.tostring(root, pretty_print=True, encoding='UTF-8', xml_declaration=True)
        
        response = self.request(xml_data)
        if resource:
            return resource(data=response.content, requester=requester, manager=self)
        return response


class ManagerTransaction(Manager):
    api_type = 'transaction'

    def send(self, command, params=None, requester=None, api_type=None, resource=None):
        if not params:
            params = requester.translated_data

        root = etree.Element('transaction-request')

        etree.SubElement(root, 'version').text = self.api_version

        verification = etree.SubElement(root, 'verification')
        etree.SubElement(verification, 'merchantId').text = self.maxid
        etree.SubElement(verification, 'merchantKey').text = self.api_key

        order = etree.SubElement(root, 'order')

        command = etree.SubElement(order, command)
       
        for key, value in params:
            create_element_recursively(command, key).text = str(value)

        xml_data = etree.tostring(root, pretty_print=True, encoding='UTF-8', xml_declaration=True)
        
        response = self.request(xml_data)
        if resource:
            return resource(data=response.content, requester=requester, manager=self)
        return response


class ManagerRapi(Manager):
    api_type = 'rapi'

    def send(self, command, params=None, requester=None, api_type=None, resource=None):
        if not params:
            params = requester.translated_data

        root = etree.Element('rapi-request')

        etree.SubElement(root, 'version').text = self.api_version

        verification = etree.SubElement(root, 'verification')
        etree.SubElement(verification, 'merchantId').text = self.maxid
        etree.SubElement(verification, 'merchantKey').text = self.api_key

        etree.SubElement(root, 'command').text = command

        request = etree.SubElement(root, 'request')

        for key, value in params:
            create_element_recursively(request, key).text = value

        xml_data = etree.tostring(root, pretty_print=True, encoding='UTF-8', xml_declaration=True)
        response = self.request(xml_data)
                
        if resource:
            result = resource(data=response.content, requester=requester, manager=self)
            return result
        return response



# coding: utf-8
from io import BytesIO
from maxipago.utils import etree
from maxipago.resources.base import Resource
from maxipago.exceptions import PaymentException


class PaymentResource(Resource):

    def process(self):
        self.approved = False
        self.authorized = False
        self.captured = False
    
        tree = etree.parse(BytesIO(self.data))
        error_code = tree.find('errorCode')
        if error_code is not None and error_code.text != '0':
            error_message = tree.find('errorMsg').text
            raise PaymentException(message=error_message)
        
        processor_code = tree.find('processorCode')
        
        if processor_code.text is not None and processor_code.text.lower() == 'a':
            self.approved = True

        if self.approved:
            response_message = tree.find('transactionID')

        fields = [
            ('transactionID', 'transaction_id'),
            ('authCode', 'auth_code'),
            ('orderID', 'order_id'),
            ('referenceNum', 'reference_num'),
            ('transactionTimestamp', 'transaction_timestamp'),
            ('boletoUrl', 'boleto_url'),
            ('processorCode', 'processor_code'),
            ('responseCode', 'response_code'),
            ('processorTransactionID', 'processor_transaction_id'),
            ('processorReferenceNumber', 'processor_reference_number'),
            ('errorMessage', 'error_message'),
            ('avsResponseCode', 'avs_response_code'),
            ('processorMessage', 'processor_message'),
            ('processorName', 'processor_name'),
            ('onlineDebitURL', 'online_debit_url'),
            ('authenticationURL', 'authentication_url'),
            ('creditCardBin', 'credit_card_bin'),
            ('creditCardLast4', 'credit_card_last_4'),
            ('creditCardCountry', 'credit_card_coutry'),
            ('creditCardScheme', 'credit_card_scheme'),
        ]

        for f_name, f_translated in fields:
            field = tree.find(f_name)
            if field is not None:
                setattr(self, f_translated, field.text)

        response_message = tree.find('responseMessage')
        
        if response_message is not None and response_message.text:
            response_message = response_message.text.lower()
            self.response_message = response_message
    
            if response_message == 'authorized':
                self.authorized = True
            elif response_message == 'captured':
                self.authorized = True
                self.captured = True
            elif response_message == 'issued':
                self.authorized = True



"""maxiPago python integration"""
# :copyright: (c) 2013 by Stored (www.stored.com.br).
# :license:   BSD, see LICENSE for more details.

VERSION = (1, 2, 1)
__version__ = '.'.join(map(str, VERSION[0:3])) + ''.join(VERSION[3:])
__author__ = 'Stored'
__contact__ = 'contato@stored.com.br'
__docformat__ = 'restructuredtext'
__license__ = 'MIT'


from maxipago.client import Maxipago



# coding: utf-8
from maxipago.exceptions import ValidationError


class Requester(object):
    def __init__(self, fields, data):
        self.fields = fields
        self.data = data

        self.full_clean()

    def _clean_fields(self):
        for field_name, field_options in self.fields:
            if field_name in self.cleaned_data:
                if hasattr(self, 'clean_{0}'.format(field_name)):
                    value = getattr(self, 'clean_{0}'.format(field_name))
                else:
                    value = self.data.get(field_name)

                self.cleaned_data[field_name] = value

    def _translate_data(self):
        self.translated_data = []
        for field_name, field_options in self.fields:
            if field_name in self.cleaned_data:
                translated_name = field_options.get('translated_name', field_name)
                self.translated_data.append((translated_name, self.cleaned_data.get(field_name)))

    def full_clean(self):
        self.cleaned_data = {}

        for field_name, field_options in self.fields:
            default = field_options.get('default', None)
            required = field_options.get('required', True)
            blank = field_options.get('blank', False)

            if ('default' not in field_options) and required and (field_name not in self.data):
                raise ValidationError('The field "{0}" is required.'.format(field_name))

            if field_name in self.data or 'default' in field_options:
                if not blank and not self.data.get(field_name, default):
                    raise ValidationError('The field "{0}" can not be empty.'.format(field_name))

                self.cleaned_data[field_name] = self.data.get(field_name, default)

        self._clean_fields()
        self._translate_data()



# coding: utf-8
from maxipago.managers.base import ManagerTransaction, ManagerApi
from maxipago.requesters.payment import PaymentRecurringRequester
from maxipago.resources.payment import PaymentResource
from maxipago.resources.cancel import CancelResource

class PaymentRecurringManager(ManagerTransaction):

    def add(self, **kwargs):
        fields = (
            ('processor_id', {'translated_name': 'processorID'}),
            ('reference_num', {'translated_name': 'referenceNum'}),
            ('ip_address', {'translated_name': 'ipAddress', 'required': False}),
            ('order_id', {'translated_name': 'orderID', 'required': False}),

            ('card_number', {'translated_name': 'transactionDetail/payType/creditCard/number', 'required': False}),
            ('card_expiration_month', {'translated_name': 'transactionDetail/payType/creditCard/expMonth', 'required': False}),
            ('card_expiration_year', {'translated_name': 'transactionDetail/payType/creditCard/expYear', 'required': False}),
            ('card_cvv', {'translated_name': 'transactionDetail/payType/creditCard/cvvNumber', 'required': False}),

            ('charge_total', {'translated_name': 'payment/chargeTotal'}),
            ('currency_code', {'translated_name': 'payment/currencyCode', 'required': True}),

            ('recurring_action', {'translated_name': 'recurring/action', 'default': 'new'}),
            ('recurring_start', {'translated_name': 'recurring/startDate', 'required': False}),
            ('recurring_last', {'translated_name': 'recurring/lastDate', 'required': False}),
            ('recurring_frequency', {'translated_name': 'recurring/frequency'}),
            ('recurring_period', {'translated_name': 'recurring/period'}),
            ('recurring_first_amount', {'translated_name': 'recurring/firstAmount', 'required': False}),
            ('recurring_last_amount', {'translated_name': 'recurring/lastAmount', 'required': False}),
            ('recurring_installments', {'translated_name': 'recurring/installments'}),
            ('recurring_failure_threshold', {'translated_name': 'recurring/failureThreshold', 'required': False}),
        )
        
        requester = PaymentRecurringRequester(fields, kwargs)
        resource = PaymentResource
        if kwargs['recurring_action'] == 'modify':
            resource = CancelResource
        return self.send(command='recurringPayment', requester=requester, resource=resource)

    def delete(self, **kwargs):
        fields = (
            ('order_id', {'translated_name': 'orderID'}),
        )

        requester = PaymentRecurringRequester(fields, kwargs)
        
        manager = ManagerApi(maxid=self.maxid, api_key=self.api_key, api_version=self.api_version, sandbox=self.sandbox)
        return manager.send(command='cancel-recurring', requester=requester, resource=CancelResource)
